# -*- coding: utf-8 -*-
#!/usr/bin/env python
#
# Copyright 2021 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing fusion predictions creation

"""


import os

from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module,
                                 teardown_class)


import bigmler.tests.composite_steps as composite_create
import bigmler.tests.basic_tst_prediction_steps as source_create


def setup_module():
    """Setup for the module

    """
    common_setup_module()


def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()


class TestComposite(object):

    def setup(self):
        """
            Debug information
        """
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown(self):
        """Calling generic teardown for every method

        """
        self.world = teardown_class()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)


    def test_scenario01(self):
        """
        Scenario: Successfully building a composite
            And I create a source from "<data>"
            And I check that the source is ready
            And I create an composite with the source
            And I check that the composite is ready
            And I check that the source is a composite component
            And I remove the source from the composite
            And I check that the composite is empty


        """
        print(self.test_scenario01.__doc__)
        examples = [
            ['data/iris.csv', './scenario41_01']]
        for example in examples:
            print("\nTesting with:\n", example)
            source_create.i_create_source( \
                self, data=example[0], output_dir=example[1])
            source_create.i_check_create_source(self)
            self.sources = [world.source["resource"]]
            composite_create.i_create_composite_from_sources( \
                self, sources=",".join(self.sources), output_dir=example[1])
            composite_create.i_check_create_composite(self)
            composite_create.check_sources_in_composite(self)
            composite_create.remove_sources(self, output_dir=example[1])
            composite_create.i_check_create_composite(self)
            self.source = self.sources[0]
            self.sources = []
            composite_create.check_sources_in_composite(self)
            source_create.check_source_exists(self, exists=True)

    def test_scenario02(self):
        """
        Scenario: Successfully building a composite and adding sources
            And I create a source from "<data>"
            And I check that the source is ready
            And I create an empty composite and add the source
            And I check that the composite is ready
            And I check that the source is a composite component

        """
        print(self.test_scenario02.__doc__)
        examples = [
            ['data/iris.csv', './scenario41_02']]
        for example in examples:
            print("\nTesting with:\n", example)
            source_create.i_create_source( \
                self, data=example[0], output_dir=example[1])
            source_create.i_check_create_source(self)
            self.sources = [world.source["resource"]]
            composite_create.i_create_empty_composite_and_add_source( \
                self, add_sources=",".join(self.sources),
                output_dir=example[1])
            composite_create.i_check_create_composite(self)
            composite_create.check_sources_in_composite(self)

    def test_scenario03(self):
        """
        Scenario: Successfully building a composite
            And I create a source from "<data>"
            And I check that the source is ready
            And I create an composite with the source
            And I check that the composite is ready
            And I check that the source is a composite component
            And I remove the source from the composite
            And I check that the composite is empty


        """
        print(self.test_scenario03.__doc__)
        examples = [
            ['data/iris.csv', './scenario41_03']]
        for example in examples:
            print("\nTesting with:\n", example)
            source_create.i_create_source( \
                self, data=example[0], output_dir=example[1])
            source_create.i_check_create_source(self)
            self.sources = [world.source["resource"]]
            composite_create.i_create_composite_from_sources( \
                self, sources=",".join(self.sources), output_dir=example[1])
            composite_create.i_check_create_composite(self)
            composite_create.check_sources_in_composite(self)
            composite_create.delete_sources(self, output_dir=example[1])
            composite_create.i_check_create_composite(self)
            self.source = self.sources[0]
            self.sources = []
            composite_create.check_sources_in_composite(self)
            source_create.check_source_exists(self, exists=False)

    def test_scenario04(self):
        """
        Scenario: Successfully building an images composite
            And I create a source from a "<zip>" of "<images_number>" images
            And I check that the composite is ready
            And I check that it has "<images_number>" components

        """
        print(self.test_scenario04.__doc__)
        examples = [
            ['data/images/fruits_hist.zip', './scenario41_04']]
        for example in examples:
            print("\nTesting with:\n", example)
            source_create.i_create_source( \
                self, data=example[0], output_dir=example[1])
            composite_create.i_check_create_composite(self)
            self.sources = world.source["object"].get("sources", [])
            composite_create.check_images_number_in_composite(self,
                                                              example[0])


    def test_scenario05(self):
        """
        Scenario: Successfully building an images composite from directory
            And I create a source from a "<directory>" of "<images_number>" images
            And I check that the composite is ready
            And I check that it has "<images_number>" components

        """
        print(self.test_scenario05.__doc__)
        examples = [
            ['data/images/fruits_hist/', './scenario41_05']]
        for example in examples:
            print("\nTesting with:\n", example)
            source_create.i_create_source( \
                self, data=example[0], output_dir=example[1])
            composite_create.i_check_create_composite(self)
            self.sources = world.source["object"].get("sources", [])
            composite_create.check_images_number_in_composite(self,
                                                              example[0])


    def test_scenario06(self):
        """
        Scenario: Successfully building an annotated images composite
            And I create a source from an "<annotations_file>" and an "<images_file>"
            And I check that the composite is ready
            And I check that it has "<annotation_fields>"
        """
        print(self.test_scenario06.__doc__)
        examples = [
            ['data/images/fruits_hist.zip', 'data/images/annotations.json',
             './scenario41_06']]
        for example in examples:
            print("\nTesting with:\n", example)
            composite_create.i_create_annotated_source( \
                self, images_file=example[0], annotations_file=example[1],
                output_dir=example[2])
            composite_create.i_check_create_composite(self)
            self.sources = world.source["object"].get("sources", [])
            composite_create.check_images_number_in_composite(self,
                                                              example[0])
            composite_create.check_annotation_fields(self, example[1])

    def test_scenario07(self):
        """
        Scenario: Successfully building a <annotations_language> annotated images composite
            And I create a source from an "<annotations_dir>" and an "<images_dir>"
            And I check that the composite is ready
            And I check that it has "<annotation_fields>"
        """
        print(self.test_scenario07.__doc__)
        examples = [
            ['data/images/fruits_hist', 'data/images/VOC_annotations',
             './scenario41_07', 'VOC'],
            ['data/images/YOLO_annotations', 'data/images/YOLO_annotations',
             './scenario41_07', 'YOLO']]
        for example in examples:
            print("\nTesting with:\n", example)
            composite_create.i_create_lang_annotated_source( \
                self, images_dir=example[0], annotations_dir=example[1],
                annotations_language=example[3],
                output_dir=example[2])
            composite_create.i_check_create_composite(self)
            self.sources = world.source["object"].get("sources", [])
            composite_create.check_images_number_in_composite(self,
                                                              example[0])
            composite_create.check_annotation_fields(
                self,
                os.path.join(example[2], "annotations.json"))



# -*- coding: utf-8 -*-
#
# Copyright 2021 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""BigMLer - image annotations formatting

"""

import os
import json
import sys
import xml.etree.ElementTree as ET
import re
import glob
import cv2
import ntpath


from zipfile import ZipFile

from bigml.constants import IMAGE_EXTENSIONS


FILE_ATTR = "file"
BBOXES_ATTR = "boxes"


def bigml_coco_file(annotations_dir, annotations_format,
                    output_file=None):
    """Translates from alternative annotations format, like VOC and YOLO to
    the format accepted by BigML

    """

    if output_file is None:
        output_file = os.path.join(annotations_dir, "annotations.json")
    try:
        zipname = annotations_dir.split(os.path.sep)[-1]
    except Exception:
        zipname = "annotated_images"
    filenames = voc_to_cocojson(annotations_dir, output_file) \
        if annotations_format == "VOC" else \
        yolo_to_cocojson(annotations_dir, output_file)

    zipPath = os.path.join(annotations_dir, "%s.zip" % zipname)

    if filenames:
        if not os.path.exists(zipPath):
            zipObj = ZipFile(zipPath, 'w')
            for filename in filenames:
                zipObj.write(filename)
                zipObj.close()
        with open(os.path.join(annotations_dir, "metadata.json"), "w") as \
                metafile:
            meta_info = {"description": ("Transformed file for images"
                                         " annotations in %s" %
                                         annotations_dir),
                         "images_file": zipPath,
                         "annotations": output_file}
            json.dump(meta_info, metafile)
    return metafile


def get_image_info(annotation_root):
    path = annotation_root.findtext('path')
    if path is None:
        filename = annotation_root.findtext('filename')
    else:
        filename = os.path.basename(path)
    img_name = os.path.basename(filename)
    img_id = os.path.splitext(img_name)[0]
    if isinstance(img_id, str):
        img_id = int(re.findall(r'\d+', img_id)[0])

    size = annotation_root.find('size')
    width = int(size.findtext('width'))
    height = int(size.findtext('height'))

    image_info = {
        'filename': filename,
        'height': height,
        'width': width,
        'id': img_id
    }
    return image_info


def get_coco_annotation_from_object(obj, filename, logfile, warnings):
    label = obj.findtext('name')
    bndbox = obj.find('bndbox')
    ## xmin = int(bndbox.findtext('xmin')) - 1
    xmin = int(bndbox.findtext('xmin'))
    ## ymin = int(bndbox.findtext('ymin')) - 1
    ymin = int(bndbox.findtext('ymin'))
    xmax = int(bndbox.findtext('xmax'))
    ymax = int(bndbox.findtext('ymax'))

    ## assert xmax >= xmin and ymax >= ymin, \
    ## f"Box size error !: (xmin, ymin, xmax, ymax): {xmin, ymin, xmax, ymax}"
    ## o_width = xmax - xmin
    ## o_height = ymax - ymin

    # logging possible errors
    if xmin == xmax or ymin == ymax:
        logfile.write(f"Possible bndbox error in {filename}:\n" +
                      f"  min equal to max: (xmin, ymin, xmax, ymax):" +
                      f"  {xmin, ymin, xmax, ymax}\n")
        warnings += 1
    if xmin > xmax or ymin > ymax:
        logfile.write(f"Possible bndbox error in {filename}:\n" +
                      f"  min greater than max: (xmin, ymin, xmax, ymax):" +
                      f"  {xmin, ymin, xmax, ymax}\n")
        warnings += 1

    annotation = {
        'label': label,
        'xmin': xmin,
        'ymin': ymin,
        'xmax': xmax,
        'ymax': ymax
    }
    return annotation, warnings


def yolo_to_cocojson(yolo_dir, output_jsonfile, args):

    ## assume .txt extension for yolo annotation files,
    ## may be input as args
    yolo_extension = '.txt'
    ## extensions of acceptable image files

    output_json_dict = {
        "annotations": []
    }

    output_json_array = []

    filenames = []

    logfile_name = output_jsonfile + ".log"
    with open(logfile_name, "w") as logfile:

        warnings = 0
        message = "Start converting YOLO files in " + yolo_dir + "\n"
        u.log_message(message, args.session_file, console_log=args.verbosity)
        logfile.write("\n%s" % message)

        ## Read yolo annotation txt file
        yolo_file_list = []
        for a_file in os.listdir(yolo_dir):
            if a_file.endswith(yolo_extension):
                # print(os.path.join(yolo_dir, a_file))
                logfile.write(os.path.join(yolo_dir, a_file) + "\n")
                yolo_file_list.append(os.path.join(yolo_dir, a_file))

        logfile.write("\n")
        ## Check if the yolo files (.txt) in the yolo_dir have their
        ## corresponding images, .jpg, .jpeg, .png, etc
        for yolo_filename in yolo_file_list:
            filename_base = os.path.splitext(yolo_filename)[0]
            # print("filename_base: " + filename_base)
            # print(yolo_dir + "/" + filename_base + ".*")
            image_filename = ''

            matched_files = [file for file in
                    glob.glob(filename_base + ".*") if not
                    file.endswith(yolo_extension)]
            if len(matched_files) == 0:
                print(" Warning: no image file for " + yolo_filename +"\n")
                logfile.write(" Warning: no image file for " +
                        yolo_filename +"\n")
            else:
                # print(*matched_files, sep = ", ")
                # print("matched_files length: " + str(len(matched_files)))

                ## traverse the matched file lists to find the image file,
                ## giving a warning each time a matched file is not in the
                ## image_extensions list.
                ## if there are more than one supported image file,
                ## the last one in the matched_file list is used
                for a_file in matched_files:
                    filenames.append(a_file)
                    base, ext = os.path.splitext(a_file)
                    if ext.lower() in IMAGE_EXTENSIONS:
                        image_filename = a_file
                    else:
                        warnings += 1
                        logfile.write("Warning: unknown image file for " +
                                      yolo_filename + "\n")
                        logfile.write(a_file)
                        logfile.write("\n")

            if not image_filename or image_filename.isspace():
                continue

            # print("image_filename: " + image_filename + "\n")
            logfile.write("converting for: " + image_filename + "\n")
            image_file = cv2.imread(image_filename)
            yolo_file = open(yolo_filename, "r")
            yolo_read_lines = yolo_file.readlines()
            yolo_file.close()

            ## using ntpath module to accommodate MS-Windows paths
            #image_filename_base = os.path.basename(image_filename)
            image_filename_base = ntpath.basename(image_filename)
            logfile.write("taking as filename: " + image_filename_base + "\n")

            one_image_dict = {
                ## possible args options for full path or basename
                FILE_ATTR: image_filename_base,
                BBOXES_ATTR: []
            }

            ## yolo format - (label, xc, yc, width, height)
            ## coco format - (label, xmin, ymin, xmax, ymax)
            for yolo_line in yolo_read_lines:
                one_yolo_annotation = yolo_line
                label = one_yolo_annotation.split()[0]

                x_center = float(one_yolo_annotation.split()[1])
                y_center = float(one_yolo_annotation.split()[2])
                width = float(one_yolo_annotation.split()[3])
                height = float(one_yolo_annotation.split()[4])

                float_x_center = image_file.shape[1] * x_center
                float_y_center = image_file.shape[0] * y_center
                float_width = image_file.shape[1] * width
                float_height = image_file.shape[0] * height

                int_x_center = int(float_x_center)
                int_y_center = int(float_y_center)
                int_width = int(float_width)
                int_height = int(float_height)

                round_width = round(float_width)
                round_height = round(float_height)

                ## Yolo uses normalized coordinates to specify
                ## the bonding boxes. [x_center, y_center, width, height]
                ## Because they are normalized, i.e. ratios to the width
                ## and height of the image, they stay the same when the image
                ## is enlarged or shrank. But the bounding boxes may
                ## lose pixels due to rounding.

                ## The following operations try to reverse the effect of
                ## rounding and add a pixel to the bouding box if reasonable.
                x_center_decimal = float_x_center - int(float_x_center)
                y_center_decimal = float_y_center - int(float_y_center)
                #print("xc_decimal=", x_center_decimal, " yc_decimal=",
                #        y_center_decimal, "\n")
                logfile.write("xc_decimal=" + str(x_center_decimal) +
                              " yc_decimal=" + str(y_center_decimal) + "\n")
                if x_center_decimal <= 0.25:
                    round_x_center = int(float_x_center)
                elif x_center_decimal > 0.25 and x_center_decimal <= 0.75:
                    round_x_center = int(float_x_center) + 0.5
                else:
                    round_x_center = int(float_x_center) + 1

                if y_center_decimal <= 0.25:
                    round_y_center = int(float_y_center)
                elif y_center_decimal > 0.25 and y_center_decimal <= 0.75:
                    round_y_center = int(float_y_center) + 0.5
                else:
                    round_y_center = int(float_y_center) + 1

                if round_x_center - int(round_x_center) == 0:
                    if (round(float_width) % 2) == 0:
                        proper_width = round(float_width)
                    else:
                        proper_width = round(float_width) + 1
                else:
                    if (round(float_width) % 2) == 0:
                        proper_width = round(float_width) + 1
                    else:
                        proper_width = round(float_width)

                if round_y_center - int(round_y_center) == 0:
                    if (round(float_height) % 2) == 0:
                        proper_height = round(float_height)
                    else:
                        proper_height = round(float_height) + 1
                else:
                    if (round(float_height) % 2) == 0:
                        proper_height = round(float_height) + 1
                    else:
                        proper_height = round(float_height)


                #print("xc=", x_center, " yc=", y_center, " w=", width,
                #      " h=", height, "\n")
                logfile.write("xc=" + str(x_center) + " yc=" + str(y_center) +
                            " w=" + str(width) + " h=" + str(height) + "\n")
                #print("float_xc=", float_x_center, " float_yc=", float_y_center,
                #      " float_w=", float_width, " float_h=", float_height, "\n")
                logfile.write("float_xc=" + str(float_x_center) + " float_yc=" +
                              str(float_y_center) + " float_w=" +
                              str(float_width) + " float_h=" +
                              str(float_height) + "\n")
                #print("round_xc=", round_x_center, " round_yc=", round_y_center,
                #      " round_w=", round_width, " round_h=", round_height, "\n")
                logfile.write("round_xc=" + str(round_x_center) + " round_yc=" +
                              str(round_y_center) + " round_w=" +
                              str(round_width) + " round_h=" +
                              str(round_height) + "\n")
                #print("round_xc=", round_x_center, " round_yc=", round_y_center,
                #    " proper_w=", proper_width, " proper_h=", proper_height, "\n")
                logfile.write("round_xc=" + str(round_x_center) + " round_yc=" +
                              str(round_y_center) + " proper_w=" +
                              str(proper_width) + " proper_h=" +
                              str(proper_height) + "\n")
                #print("int_xc=", int_x_center, " int_yc=", int_y_center,
                #      " int_w=", int_width, " int_h=", int_height, "\n\n")
                logfile.write("int_xc=" + str(int_x_center) + " int_yc=" +
                              str(int_y_center) + " int_w=" + str(int_width) +
                              " int_h=" + str(int_height) + "\n\n")

                ## x_min = int_x_center - int_width/2
                x_min = int(round_x_center - proper_width/2)
                ## y_min = int_y_center - int_height/2
                y_min = int(round_y_center - proper_height/2)
                ## x_max = x_min + int_width
                x_max = int(x_min + proper_width)
                ## y_max = y_min + int_height
                y_max = int(y_min + proper_height)
                annotation = {
                        'label': label,
                        'xmin': x_min,
                        'ymin': y_min,
                        'xmax': x_max,
                        'ymax': y_max
                        }
                one_image_dict[BBOXES_ATTR].append(annotation)

            # output_json_dict['annotations'].append(one_image_dict)
            output_json_array.append(one_image_dict)

    if warnings > 0:
        message = f"\nThere are {warnings} warnings, " \
                  f"see the log file {logfile_name}\n"
        u.log_message(message, args.session_file, console_log=args.verbosity)

    with open(output_jsonfile, 'w') as f:
        output_json = json.dumps(output_json_array, indent=2)
        f.write(output_json)

    return filenames


def voc_to_cocojson(voc_dir, output_jsonfile, args):
    """Translates annotations from a VOC format, where each image is associated
    to a .xml file that contains one object per associated info. It returns
    the list of images it refers to.

    """
    output_json_dict = {
        "annotations": []
    }

    output_json_array = []

    filenames = []

    annotation_file_list = []
    for file in os.listdir(voc_dir):
        if file.endswith(".xml"):
            annotation_file_list.append(os.path.join(voc_dir, file))
    logfile_name = output_jsonfile + ".log"
    with open(logfile_name, "w") as logfile:

        ## Start bounding_box_id, possible future use, may be input as args
        bndbox_id = 1
        warnings = 0
        message = "Start converting VOC files in " + voc_dir + "\n")
        u.log_message(message, args.session_file, console_log=args.verbosity)
        logfile.write("\n\n%s\n" % message)
        for a_file in annotation_file_list:
            ## Read annotation xml per image
            annotation_tree = ET.parse(a_file)
            annotation_root = annotation_tree.getroot()

            img_info = get_image_info(annotation_root)
            ## img_id for possible future use
            img_id = img_info['id']
            filename = img_info['filename']
            filenames.append(filename)

            ## possible args options for full path or basename
            logfile.write("converting for: " + filename + "\n")
            ## using ntpath module instead of os.path
            ## to accommodate MS-Windows paths
            image_filename_base = ntpath.basename(filename)
            logfile.write("taking as filename: " + image_filename_base + "\n")

            one_image_dict = {
                ## possible args options for full path or basename
                FILE_ATTR: image_filename_base,
                BBOXES_ATTR: []
            }

            for obj in annotation_root.findall('object'):
                annotation, warnings = get_coco_annotation_from_object( \
                    obj, filename, logfile, warnings)
                ## output_json_dict['annotations'].append(ann)
                ## output_json_dict['bboxes'].append(ann)
                one_image_dict[BBOXES_ATTR].append(annotation)
                bndbox_id = bndbox_id + 1

            # output_json_dict['annotations'].append(one_image_dict)
            output_json_array.append(one_image_dict)

    if warnings > 0:
        message = f"\nThere are {warnings} warnings, " \
                f"see the log file {logfile_name}\n"
        u.log_message(message, args.session_file, console_log=args.verbosity)


    with open(output_jsonfile, 'w') as f:
        output_json = json.dumps(output_json_array, indent=2)
        f.write(output_json)

    return filenames



# -*- coding: utf-8 -*-
#
# Copyright 2019-2022 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import os
import time
import json
from bigmler.tests.world import world, res_filename
from subprocess import check_call, CalledProcessError
from bigml.api import check_resource
from bigml.io import UnicodeReader
from bigmler.checkpoint import file_number_of_lines
from bigmler.utils import storage_file_name, open_mode
from bigmler.tests.common_steps import check_debug
from nose.tools import ok_, assert_equal, assert_not_equal
import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.basic_deepnet_steps as test_dn


def shell_execute(command, output, test=None, options=None,
                  test_rows=None, project=True):
    """Excute bigmler command in shell

    """
    command = check_debug(command, project=project)
    world.directory = os.path.dirname(output)
    world.folders.append(world.directory)
    try:
        retcode = check_call(command, shell=True)
        if retcode < 0:
            assert False
        else:
            if test is not None:
                world.test_lines = file_number_of_lines(test)
                if options is None or \
                        options.find('--prediction-header') == -1:
                    # test file has headers in it, so first line must be ignored
                    world.test_lines -= 1
            elif test_rows is not None:
                world.test_lines = test_rows
                if options is not None and \
                        options.find('--prediction-header') > -1:
                    world.test_lines += 1
            elif options is not None and \
                    options.find('--prediction-header') > -1:
                world.test_lines += 1
            world.output = output
            assert True
    except (OSError, CalledProcessError, IOError) as exc:
        assert False, str(exc)


#@step(r'I create BigML resources using model to test "(.*)" and log predictions in "(.*)"')
def i_create_fs_resources_from_model(step, test=None, output=None):
    ok_(test is not None and output is not None)
    test = res_filename(test)

    models = [world.model["resource"], world.deepnet["resource"]]
    command = ("bigmler fusion --fusion-models " +
               ",".join(models) + " --test \"" +
               test + "\" --store --output " +
               output)
    shell_execute(command, output, test=test)


#@step(r'I create BigML resources using model to test "(.*)" as batch prediction and log predictions in "(.*)"')
def i_create_fs_resources_from_model_remote(step, test=None, output=None):
    ok_(test is not None and output is not None)
    test = res_filename(test)

    models = [world.model["resource"], world.deepnet["resource"]]
    command = ("bigmler fusion --fusion-models " +
               ",".join(models) + " --test \"" + test +
               "\" --store --remote --output " +
               output)
    shell_execute(command, output, test=test)

#@step(r'I create BigML resources using model to test "(.*)" as batch prediction with output format "(.*)" and log predictions in "(.*)"')
def i_create_fs_resources_from_model_remote_with_options(step, test=None, output=None, options_file=None):
    ok_(test is not None and output is not None and options_file is not None)
    test = res_filename(test)
    options_file = res_filename(options_file)

    models = [world.model["resource"], world.deepnet["resource"]]
    command = ("bigmler fusion --fusion-models " +
               ",".join(models) + " --test \"" + test +
               "\" --batch-prediction-attributes " + options_file +
               " --store --remote --output " +
               output)
    shell_execute(command, output, test=test)

#@step(r'I create BigML resources using model to test "(.*)" and an evaluation and log predictions in "(.*)"')
def i_create_fs_resources_from_mode_and_evaluate(step, output=None):
    ok_(output is not None)
    models = [world.model["resource"], world.deepnet["resource"]]
    command = ("bigmler fusion --fusion-models " +
               ",".join(models) + " --test-dataset \"" +
               world.dataset["resource"] +
               "\" --evaluate --store --remote --output " +
               output)
    shell_execute(command, output)

#@step(r'I check that the fusion model has been created')
def i_check_create_fusion(step):
    fs_file = "%s%sfusions" % (world.directory, os.sep)
    try:
        fs_file = open(fs_file, "r")
        fs = check_resource(fs_file.readline().strip(),
                            world.api.get_fusion)
        world.fusions.append(fs['resource'])
        world.fusion = fs
        fs_file.close()
    except Exception as exc:
        assert False, str(exc)

def setup_for_fusion(step, train=None, output_dir=None):
    train = res_filename(train)
    command = ("bigmler --train \"" +
               train + "\" --store --output-dir " +
               output_dir)
    shell_execute(command, "%s/predictions" % output_dir)
    test_pred.i_check_create_source(step)
    test_pred.i_check_create_dataset(step)
    test_pred.i_check_create_model(step)

    command = ("bigmler deepnet --dataset \"" +
               world.dataset["resource"] + "\" --store --output-dir " +
               output_dir)
    shell_execute(command, "%s/predictions" % output_dir)
    test_dn.i_check_create_dn_model(step)

#@step(r'I create BigML fusion resources uploading train "(.*)" file to evaluate and log evaluation in "([^"]*)"$')
def i_create_all_fs_resources_to_evaluate(step, train=None, test=None, output=None):
    models = [world.model["resource"], world.deepnet["resource"]]
    command = ("bigmler fusion --fusion-models " + ",".join(models) +
               " --test \"" +
               test +
               "\" --evaluate" +
               " --store --output " + output)
    shell_execute(command, output)



# -*- coding: utf-8 -*-
#
# Copyright 2021 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import os
import time
import json
import zipfile
import glob

from bigmler.tests.world import world, res_filename
from subprocess import check_call, CalledProcessError
from bigml.api import check_resource
from bigml.util import filter_by_extension
from bigml.fields import Fields
from bigml.constants import IMAGE_EXTENSIONS
from nose.tools import ok_, assert_equal, assert_not_equal, assert_almost_equal
from bigmler.tests.common_steps import check_debug
from bigmler.utils import read_sources
from bigmler.processing.annotations import fields_from_annotations, \
    labels_from_annotations


def shell_execute(command, output, test=None, options=None,
                  test_rows=None, project=True):
    """Excute bigmler command in shell

    """
    command = check_debug(command, project=project)
    world.directory = os.path.dirname(output)
    world.folders.append(world.directory)
    try:
        retcode = check_call(command, shell=True)
        ok_(retcode >= 0)
        if test is not None:
            world.test_lines = file_number_of_lines(test)
            if options is None or \
                    options.find('--prediction-header') == -1:
                # test file has headers in it, so first line must be ignored
                world.test_lines -= 1
        elif test_rows is not None:
            world.test_lines = test_rows
            if options is not None and \
                    options.find('--prediction-header') > -1:
                world.test_lines += 1
        elif options is not None and \
                options.find('--prediction-header') > -1:
            world.test_lines += 1
        world.output = output
    except (OSError, CalledProcessError, IOError) as exc:
        assert False, str(exc)

#@step(r'I create BigML composite from a list of sources and
#        log results in "(.*)"')
def i_create_composite_from_sources(step, sources=None, output_dir=None):
    ok_(sources is not None and output_dir is not None)
    command = ("bigmler source --sources " + sources +
               " --store --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "txt.tmp"))

#@step(r'I create empty BigML composite and then add list of sources and
#        log results in "(.*)"')
def i_create_empty_composite_and_add_source(step, add_sources=None,
                                            output_dir=None):
    ok_(add_sources is not None and output_dir is not None)
    command = ("bigmler source --sources \"\" --add-sources " + add_sources +
               " --store --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "txt.tmp"))

#@step(r'I remove the sources from a BigML composite and
#        log results in "(.*)"')
def remove_sources(step, output_dir=None):
    ok_(output_dir is not None)
    command = ("bigmler source --source " + world.source["resource"] +
               " --remove-sources " + ",".join(step.sources) +
               " --store --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "txt.tmp"))


#@step(r'I delete the sources from a BigML composite and
#        log results in "(.*)"')
def delete_sources(step, output_dir=None):
    ok_(output_dir is not None)
    command = ("bigmler source --source " + world.source["resource"] +
               " --delete-sources " + ",".join(step.sources) +
               " --store --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "txt.tmp"))


#@step(r'I create empty BigML composite and then add list of sources and
#        log results in "(.*)"')
def i_create_empty_composite_and_add_source(step, add_sources=None,
                                            output_dir=None):
    ok_(add_sources is not None and output_dir is not None)
    command = ("bigmler source --sources \"\" --add-sources " + add_sources +
               " --store --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "txt.tmp"))

#@step(r'I create  BigML composite for an "<annotations_file>" and an
# "<images_file>" and log results in "(.*)"')
def i_create_annotated_source(step, annotations_file=None,
                                    images_file=None,
                                    output_dir=None):
    ok_(annotations_file is not None and images_file is not None and
        output_dir is not None)
    command = ("bigmler source --annotations-file " + annotations_file +
               " --data " + images_file +
               " --store --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "txt.tmp"))


#@step(r'I create  BigML composite for a "<annotations_language>"
# "<annotations_dir>" and an
# "<images_dir>" and log results in "(.*)"')
def i_create_lang_annotated_source(step, annotations_dir=None,
                                        images_dir=None,
                                        annotations_language=None,
                                        output_dir=None):
    ok_(annotations_dir is not None and images_dir is not None and
        annotations_language is not None and output_dir is not None)
    command = ("bigmler source --annotations-dir " + annotations_dir +
               " --data " + images_dir + " --annotations-language " +
               annotations_language +
               " --store --output-dir " + output_dir)
    shell_execute(command, os.path.join(output_dir, "txt.tmp"))


#@step(r'I check that the sources are in the composite')
def check_sources_in_composite(step):
    assert_equal(step.sources, ["source/%s" % source for source in
                                  world.source["object"].get("sources", [])])

#@step(r'I check that the composite has been created$')
def i_check_create_composite(step):
    source_file = os.path.join(world.directory, "source")
    try:
        last_source = read_sources(source_file)
        source = check_resource(
            last_source.strip(), world.api.get_source)
        if source['resource'] not in world.composites:
            world.composites.append(source['resource'])
        world.source = source
    except Exception as exc:
        assert False, str(exc)

#@step(r'I check the number of sources in the composite is <images_number>')
def check_images_number_in_composite(step, zip_file):
    sources = []
    if os.path.isdir(zip_file):
        sources = filter_by_extension(glob.glob(os.path.join(zip_file, "**"),
                                                recursive=True),
                                      IMAGE_EXTENSIONS)
    else:
        zip = zipfile.ZipFile(zip_file)
        sources = filter_by_extension(zip.namelist(), IMAGE_EXTENSIONS)
    assert_equal(len(sources), len(step.sources))

def check_annotation_fields(step, annotations_file):
    fields = fields_from_annotations(annotations_file)
    field_labels = labels_from_annotations(annotations_file)
    source_fields = Fields(world.source)
    field_names = list(source_fields.fields_by_name.keys())
    for field in fields:
        assert field["name"] in field_names
        field_labels[field["name"]].sort()
        source_fields.fields[
            source_fields.fields_by_name[
            field["name"]]]["values"].sort()
        assert field_labels[
            field["name"]] == source_fields.fields[
            source_fields.fields_by_name[
            field["name"]]]["values"]
